@@ -50,11 +50,11 @@ class RasterioArrayWrapper(BackendArray):
         return self._shape
 
     def _get_indexer(self, key):
-        """ Get indexer for rasterio array.
+        """Get indexer for rasterio array.
 
-        Parameter
-        ---------
-        key: tuple of int
+        Parameters
+        ----------
+        key : tuple of int
 
         Returns
         -------
@@ -63,7 +63,7 @@ class RasterioArrayWrapper(BackendArray):
         squeeze_axis: axes to be squeezed
         np_ind: indexer for loaded numpy array
 
-        See also
+        See Also
         --------
         indexing.decompose_indexer
         """
@@ -180,7 +180,6 @@ def open_rasterio(filename, parse_coordinates=None, chunks=None, cache=None, loc
         nx, ny = da.sizes['x'], da.sizes['y']
         x, y = np.meshgrid(np.arange(nx)+0.5, np.arange(ny)+0.5) * transform
 
-
     Parameters
     ----------
     filename : str, rasterio.DatasetReader, or rasterio.WarpedVRT
@@ -361,6 +360,6 @@ def open_rasterio(filename, parse_coordinates=None, chunks=None, cache=None, loc
         result = result.chunk(chunks, name_prefix=name_prefix, token=token)
 
     # Make the file closeable
-    result._file_obj = manager
+    result.set_close(manager.close)
 
     return result
