@@ -1,14 +1,28 @@
-from io import BytesIO
+import io
+import os
 
 import numpy as np
 
 from ..core.indexing import NumpyIndexingAdapter
-from ..core.utils import Frozen, FrozenDict
+from ..core.utils import Frozen, FrozenDict, close_on_error, read_magic_number
 from ..core.variable import Variable
-from .common import BackendArray, WritableCFDataStore
+from .common import (
+    BACKEND_ENTRYPOINTS,
+    BackendArray,
+    BackendEntrypoint,
+    WritableCFDataStore,
+)
 from .file_manager import CachingFileManager, DummyFileManager
 from .locks import ensure_lock, get_write_lock
 from .netcdf3 import encode_nc3_attr_value, encode_nc3_variable, is_valid_nc3_name
+from .store import StoreBackendEntrypoint
+
+try:
+    import scipy.io
+
+    has_scipy = True
+except ModuleNotFoundError:
+    has_scipy = False
 
 
 def _decode_string(s):
@@ -59,8 +73,6 @@ class ScipyArrayWrapper(BackendArray):
 def _open_scipy_netcdf(filename, mode, mmap, version):
     import gzip
 
-    import scipy.io
-
     # if the string ends with .gz, then gunzip and open as netcdf file
     if isinstance(filename, str) and filename.endswith(".gz"):
         try:
@@ -76,7 +88,7 @@ def _open_scipy_netcdf(filename, mode, mmap, version):
 
     if isinstance(filename, bytes) and filename.startswith(b"CDF"):
         # it's a NetCDF3 bytestring
-        filename = BytesIO(filename)
+        filename = io.BytesIO(filename)
 
     try:
         return scipy.io.netcdf_file(filename, mode=mode, mmap=mmap, version=version)
@@ -218,3 +230,56 @@ class ScipyDataStore(WritableCFDataStore):
 
     def close(self):
         self._manager.close()
+
+
+class ScipyBackendEntrypoint(BackendEntrypoint):
+    def guess_can_open(self, store_spec):
+        try:
+            return read_magic_number(store_spec).startswith(b"CDF")
+        except TypeError:
+            pass
+
+        try:
+            _, ext = os.path.splitext(store_spec)
+        except TypeError:
+            return False
+        return ext in {".nc", ".nc4", ".cdf", ".gz"}
+
+    def open_dataset(
+        self,
+        filename_or_obj,
+        mask_and_scale=True,
+        decode_times=True,
+        concat_characters=True,
+        decode_coords=True,
+        drop_variables=None,
+        use_cftime=None,
+        decode_timedelta=None,
+        mode="r",
+        format=None,
+        group=None,
+        mmap=None,
+        lock=None,
+    ):
+
+        store = ScipyDataStore(
+            filename_or_obj, mode=mode, format=format, group=group, mmap=mmap, lock=lock
+        )
+
+        store_entrypoint = StoreBackendEntrypoint()
+        with close_on_error(store):
+            ds = store_entrypoint.open_dataset(
+                store,
+                mask_and_scale=mask_and_scale,
+                decode_times=decode_times,
+                concat_characters=concat_characters,
+                decode_coords=decode_coords,
+                drop_variables=drop_variables,
+                use_cftime=use_cftime,
+                decode_timedelta=decode_timedelta,
+            )
+        return ds
+
+
+if has_scipy:
+    BACKEND_ENTRYPOINTS["scipy"] = ScipyBackendEntrypoint
