@@ -1,5 +1,6 @@
 import datetime as dt
 import warnings
+from distutils.version import LooseVersion
 from functools import partial
 from numbers import Number
 from typing import Any, Callable, Dict, Hashable, Sequence, Union
@@ -10,8 +11,9 @@ import pandas as pd
 from . import utils
 from .common import _contains_datetime_like_objects, ones_like
 from .computation import apply_ufunc
-from .duck_array_ops import dask_array_type, datetime_to_numeric, timedelta_to_numeric
+from .duck_array_ops import datetime_to_numeric, timedelta_to_numeric
 from .options import _get_keep_attrs
+from .pycompat import is_duck_dask_array
 from .utils import OrderedSet, is_scalar
 from .variable import Variable, broadcast_variables
 
@@ -44,8 +46,7 @@ def _get_nan_block_lengths(obj, dim: Hashable, index: Variable):
 
 
 class BaseInterpolator:
-    """Generic interpolator class for normalizing interpolation methods
-    """
+    """Generic interpolator class for normalizing interpolation methods"""
 
     cons_kwargs: Dict[str, Any]
     call_kwargs: Dict[str, Any]
@@ -153,7 +154,7 @@ class ScipyInterpolator(BaseInterpolator):
             yi,
             kind=self.method,
             fill_value=fill_value,
-            bounds_error=False,
+            bounds_error=bounds_error,
             assume_sorted=assume_sorted,
             copy=copy,
             **self.cons_kwargs,
@@ -195,8 +196,7 @@ class SplineInterpolator(BaseInterpolator):
 
 
 def _apply_over_vars_with_dim(func, self, dim=None, **kwargs):
-    """Wrapper for datasets
-    """
+    """Wrapper for datasets"""
     ds = type(self)(coords=self.coords, attrs=self.attrs)
 
     for name, var in self.data_vars.items():
@@ -216,20 +216,20 @@ def get_clean_interp_index(
     Parameters
     ----------
     arr : DataArray
-      Array to interpolate or fit to a curve.
+        Array to interpolate or fit to a curve.
     dim : str
-      Name of dimension along which to fit.
+        Name of dimension along which to fit.
     use_coordinate : str or bool
-      If use_coordinate is True, the coordinate that shares the name of the
-      dimension along which interpolation is being performed will be used as the
-      x values. If False, the x values are set as an equally spaced sequence.
+        If use_coordinate is True, the coordinate that shares the name of the
+        dimension along which interpolation is being performed will be used as the
+        x values. If False, the x values are set as an equally spaced sequence.
     strict : bool
-      Whether to raise errors if the index is either non-unique or non-monotonic (default).
+        Whether to raise errors if the index is either non-unique or non-monotonic (default).
 
     Returns
     -------
     Variable
-      Numerical values for the x-coordinates.
+        Numerical values for the x-coordinates.
 
     Notes
     -----
@@ -303,8 +303,7 @@ def interp_na(
     keep_attrs: bool = None,
     **kwargs,
 ):
-    """Interpolate values according to different methods.
-    """
+    """Interpolate values according to different methods."""
     from xarray.coding.cftimeindex import CFTimeIndex
 
     if dim is None:
@@ -438,6 +437,16 @@ def bfill(arr, dim=None, limit=None):
     ).transpose(*arr.dims)
 
 
+def _import_interpolant(interpolant, method):
+    """Import interpolant from scipy.interpolate."""
+    try:
+        from scipy import interpolate
+
+        return getattr(interpolate, interpolant)
+    except ImportError as e:
+        raise ImportError(f"Interpolation with method {method} requires scipy.") from e
+
+
 def _get_interpolator(method, vectorizeable_only=False, **kwargs):
     """helper function to select the appropriate interpolator class
 
@@ -460,12 +469,6 @@ def _get_interpolator(method, vectorizeable_only=False, **kwargs):
         "akima",
     ]
 
-    has_scipy = True
-    try:
-        from scipy import interpolate
-    except ImportError:
-        has_scipy = False
-
     # prioritize scipy.interpolate
     if (
         method == "linear"
@@ -476,32 +479,29 @@ def _get_interpolator(method, vectorizeable_only=False, **kwargs):
         interp_class = NumpyInterpolator
 
     elif method in valid_methods:
-        if not has_scipy:
-            raise ImportError("Interpolation with method `%s` requires scipy" % method)
-
         if method in interp1d_methods:
             kwargs.update(method=method)
             interp_class = ScipyInterpolator
         elif vectorizeable_only:
             raise ValueError(
-                "{} is not a vectorizeable interpolator. "
-                "Available methods are {}".format(method, interp1d_methods)
+                f"{method} is not a vectorizeable interpolator. "
+                f"Available methods are {interp1d_methods}"
             )
         elif method == "barycentric":
-            interp_class = interpolate.BarycentricInterpolator
+            interp_class = _import_interpolant("BarycentricInterpolator", method)
         elif method == "krog":
-            interp_class = interpolate.KroghInterpolator
+            interp_class = _import_interpolant("KroghInterpolator", method)
         elif method == "pchip":
-            interp_class = interpolate.PchipInterpolator
+            interp_class = _import_interpolant("PchipInterpolator", method)
         elif method == "spline":
             kwargs.update(method=method)
             interp_class = SplineInterpolator
         elif method == "akima":
-            interp_class = interpolate.Akima1DInterpolator
+            interp_class = _import_interpolant("Akima1DInterpolator", method)
         else:
-            raise ValueError("%s is not a valid scipy interpolator" % method)
+            raise ValueError(f"{method} is not a valid scipy interpolator")
     else:
-        raise ValueError("%s is not a valid interpolator" % method)
+        raise ValueError(f"{method} is not a valid interpolator")
 
     return interp_class, kwargs
 
@@ -513,18 +513,13 @@ def _get_interpolator_nd(method, **kwargs):
     """
     valid_methods = ["linear", "nearest"]
 
-    try:
-        from scipy import interpolate
-    except ImportError:
-        raise ImportError("Interpolation with method `%s` requires scipy" % method)
-
     if method in valid_methods:
         kwargs.update(method=method)
-        interp_class = interpolate.interpn
+        interp_class = _import_interpolant("interpn", method)
     else:
         raise ValueError(
-            "%s is not a valid interpolator for interpolating "
-            "over multiple dimensions." % method
+            f"{method} is not a valid interpolator for interpolating "
+            "over multiple dimensions."
         )
 
     return interp_class, kwargs
@@ -544,24 +539,25 @@ def _get_valid_fill_mask(arr, dim, limit):
     ) <= limit
 
 
-def _assert_single_chunk(var, axes):
-    for axis in axes:
-        if len(var.chunks[axis]) > 1 or var.chunks[axis][0] < var.shape[axis]:
-            raise NotImplementedError(
-                "Chunking along the dimension to be interpolated "
-                "({}) is not yet supported.".format(axis)
-            )
-
-
 def _localize(var, indexes_coords):
-    """ Speed up for linear and nearest neighbor method.
+    """Speed up for linear and nearest neighbor method.
     Only consider a subspace that is needed for the interpolation
     """
     indexes = {}
     for dim, [x, new_x] in indexes_coords.items():
+        if np.issubdtype(new_x.dtype, np.datetime64) and LooseVersion(
+            np.__version__
+        ) < LooseVersion("1.18"):
+            # np.nanmin/max changed behaviour for datetime types in numpy 1.18,
+            # see https://github.com/pydata/xarray/pull/3924/files
+            minval = np.min(new_x.values)
+            maxval = np.max(new_x.values)
+        else:
+            minval = np.nanmin(new_x.values)
+            maxval = np.nanmax(new_x.values)
         index = x.to_index()
-        imin = index.get_loc(np.min(new_x.values), method="nearest")
-        imax = index.get_loc(np.max(new_x.values), method="nearest")
+        imin = index.get_loc(minval, method="nearest")
+        imax = index.get_loc(maxval, method="nearest")
 
         indexes[dim] = slice(max(imin - 2, 0), imax + 2)
         indexes_coords[dim] = (x[indexes[dim]], new_x)
@@ -569,7 +565,7 @@ def _localize(var, indexes_coords):
 
 
 def _floatize_x(x, new_x):
-    """ Make x and new_x float.
+    """Make x and new_x float.
     This is particulary useful for datetime dtype.
     x, new_x: tuple of np.ndarray
     """
@@ -589,20 +585,20 @@ def _floatize_x(x, new_x):
 
 
 def interp(var, indexes_coords, method, **kwargs):
-    """ Make an interpolation of Variable
+    """Make an interpolation of Variable
 
     Parameters
     ----------
-    var: Variable
-    index_coords:
+    var : Variable
+    indexes_coords
         Mapping from dimension name to a pair of original and new coordinates.
         Original coordinates should be sorted in strictly ascending order.
         Note that all the coordinates should be Variable objects.
-    method: string
+    method : string
         One of {'linear', 'nearest', 'zero', 'slinear', 'quadratic',
         'cubic'}. For multidimensional interpolation, only
         {'linear', 'nearest'} can be used.
-    **kwargs:
+    **kwargs
         keyword arguments to be passed to scipy.interpolate
 
     Returns
@@ -617,49 +613,42 @@ def interp(var, indexes_coords, method, **kwargs):
     if not indexes_coords:
         return var.copy()
 
-    # simple speed up for the local interpolation
-    if method in ["linear", "nearest"]:
-        var, indexes_coords = _localize(var, indexes_coords)
-
     # default behavior
     kwargs["bounds_error"] = kwargs.get("bounds_error", False)
 
-    # check if the interpolation can be done in orthogonal manner
-    if (
-        len(indexes_coords) > 1
-        and method in ["linear", "nearest"]
-        and all(dest[1].ndim == 1 for dest in indexes_coords.values())
-        and len(set([d[1].dims[0] for d in indexes_coords.values()]))
-        == len(indexes_coords)
-    ):
-        # interpolate sequentially
-        for dim, dest in indexes_coords.items():
-            var = interp(var, {dim: dest}, method, **kwargs)
-        return var
+    result = var
+    # decompose the interpolation into a succession of independant interpolation
+    for indexes_coords in decompose_interp(indexes_coords):
+        var = result
 
-    # target dimensions
-    dims = list(indexes_coords)
-    x, new_x = zip(*[indexes_coords[d] for d in dims])
-    destination = broadcast_variables(*new_x)
+        # simple speed up for the local interpolation
+        if method in ["linear", "nearest"]:
+            var, indexes_coords = _localize(var, indexes_coords)
 
-    # transpose to make the interpolated axis to the last position
-    broadcast_dims = [d for d in var.dims if d not in dims]
-    original_dims = broadcast_dims + dims
-    new_dims = broadcast_dims + list(destination[0].dims)
-    interped = interp_func(
-        var.transpose(*original_dims).data, x, destination, method, kwargs
-    )
+        # target dimensions
+        dims = list(indexes_coords)
+        x, new_x = zip(*[indexes_coords[d] for d in dims])
+        destination = broadcast_variables(*new_x)
 
-    result = Variable(new_dims, interped, attrs=var.attrs)
+        # transpose to make the interpolated axis to the last position
+        broadcast_dims = [d for d in var.dims if d not in dims]
+        original_dims = broadcast_dims + dims
+        new_dims = broadcast_dims + list(destination[0].dims)
+        interped = interp_func(
+            var.transpose(*original_dims).data, x, destination, method, kwargs
+        )
 
-    # dimension of the output array
-    out_dims = OrderedSet()
-    for d in var.dims:
-        if d in dims:
-            out_dims.update(indexes_coords[d][1].dims)
-        else:
-            out_dims.add(d)
-    return result.transpose(*tuple(out_dims))
+        result = Variable(new_dims, interped, attrs=var.attrs)
+
+        # dimension of the output array
+        out_dims = OrderedSet()
+        for d in var.dims:
+            if d in dims:
+                out_dims.update(indexes_coords[d][1].dims)
+            else:
+                out_dims.add(d)
+        result = result.transpose(*out_dims)
+    return result
 
 
 def interp_func(var, x, new_x, method, kwargs):
@@ -669,17 +658,17 @@ def interp_func(var, x, new_x, method, kwargs):
 
     Parameters
     ----------
-    var: np.ndarray or dask.array.Array
+    var : np.ndarray or dask.array.Array
         Array to be interpolated. The final dimension is interpolated.
-    x: a list of 1d array.
+    x : a list of 1d array.
         Original coordinates. Should not contain NaN.
-    new_x: a list of 1d array
+    new_x : a list of 1d array
         New coordinates. Should not contain NaN.
-    method: string
+    method : string
         {'linear', 'nearest', 'zero', 'slinear', 'quadratic', 'cubic'} for
         1-dimensional interpolation.
         {'linear', 'nearest'} for multidimensional interpolation
-    **kwargs:
+    **kwargs
         Optional keyword arguments to be passed to scipy.interpolator
 
     Returns
@@ -687,8 +676,8 @@ def interp_func(var, x, new_x, method, kwargs):
     interpolated: array
         Interpolated array
 
-    Note
-    ----
+    Notes
+    -----
     This requiers scipy installed.
 
     See Also
@@ -703,24 +692,61 @@ def interp_func(var, x, new_x, method, kwargs):
     else:
         func, kwargs = _get_interpolator_nd(method, **kwargs)
 
-    if isinstance(var, dask_array_type):
+    if is_duck_dask_array(var):
         import dask.array as da
 
-        _assert_single_chunk(var, range(var.ndim - len(x), var.ndim))
-        chunks = var.chunks[: -len(x)] + new_x[0].shape
-        drop_axis = range(var.ndim - len(x), var.ndim)
-        new_axis = range(var.ndim - len(x), var.ndim - len(x) + new_x[0].ndim)
-        return da.map_blocks(
-            _interpnd,
+        nconst = var.ndim - len(x)
+
+        out_ind = list(range(nconst)) + list(range(var.ndim, var.ndim + new_x[0].ndim))
+
+        # blockwise args format
+        x_arginds = [[_x, (nconst + index,)] for index, _x in enumerate(x)]
+        x_arginds = [item for pair in x_arginds for item in pair]
+        new_x_arginds = [
+            [_x, [var.ndim + index for index in range(_x.ndim)]] for _x in new_x
+        ]
+        new_x_arginds = [item for pair in new_x_arginds for item in pair]
+
+        args = (
             var,
-            x,
-            new_x,
-            func,
-            kwargs,
-            dtype=var.dtype,
-            chunks=chunks,
-            new_axis=new_axis,
-            drop_axis=drop_axis,
+            range(var.ndim),
+            *x_arginds,
+            *new_x_arginds,
+        )
+
+        _, rechunked = da.unify_chunks(*args)
+
+        args = tuple([elem for pair in zip(rechunked, args[1::2]) for elem in pair])
+
+        new_x = rechunked[1 + (len(rechunked) - 1) // 2 :]
+
+        new_axes = {
+            var.ndim + i: new_x[0].chunks[i]
+            if new_x[0].chunks is not None
+            else new_x[0].shape[i]
+            for i in range(new_x[0].ndim)
+        }
+
+        # if usefull, re-use localize for each chunk of new_x
+        localize = (method in ["linear", "nearest"]) and (new_x[0].chunks is not None)
+
+        # scipy.interpolate.interp1d always forces to float.
+        # Use the same check for blockwise as well:
+        if not issubclass(var.dtype.type, np.inexact):
+            dtype = np.float_
+        else:
+            dtype = var.dtype
+
+        return da.blockwise(
+            _dask_aware_interpnd,
+            out_ind,
+            *args,
+            interp_func=func,
+            interp_kwargs=kwargs,
+            localize=localize,
+            concatenate=True,
+            dtype=dtype,
+            new_axes=new_axes,
         )
 
     return _interpnd(var, x, new_x, func, kwargs)
@@ -751,3 +777,67 @@ def _interpnd(var, x, new_x, func, kwargs):
     # move back the interpolation axes to the last position
     rslt = rslt.transpose(range(-rslt.ndim + 1, 1))
     return rslt.reshape(rslt.shape[:-1] + new_x[0].shape)
+
+
+def _dask_aware_interpnd(var, *coords, interp_func, interp_kwargs, localize=True):
+    """Wrapper for `_interpnd` through `blockwise`
+
+    The first half arrays in `coords` are original coordinates,
+    the other half are destination coordinates
+    """
+    n_x = len(coords) // 2
+    nconst = len(var.shape) - n_x
+
+    # _interpnd expect coords to be Variables
+    x = [Variable([f"dim_{nconst + dim}"], _x) for dim, _x in enumerate(coords[:n_x])]
+    new_x = [
+        Variable([f"dim_{len(var.shape) + dim}" for dim in range(len(_x.shape))], _x)
+        for _x in coords[n_x:]
+    ]
+
+    if localize:
+        # _localize expect var to be a Variable
+        var = Variable([f"dim_{dim}" for dim in range(len(var.shape))], var)
+
+        indexes_coords = {_x.dims[0]: (_x, _new_x) for _x, _new_x in zip(x, new_x)}
+
+        # simple speed up for the local interpolation
+        var, indexes_coords = _localize(var, indexes_coords)
+        x, new_x = zip(*[indexes_coords[d] for d in indexes_coords])
+
+        # put var back as a ndarray
+        var = var.data
+
+    return _interpnd(var, x, new_x, interp_func, interp_kwargs)
+
+
+def decompose_interp(indexes_coords):
+    """Decompose the interpolation into a succession of independant interpolation keeping the order"""
+
+    dest_dims = [
+        dest[1].dims if dest[1].ndim > 0 else [dim]
+        for dim, dest in indexes_coords.items()
+    ]
+    partial_dest_dims = []
+    partial_indexes_coords = {}
+    for i, index_coords in enumerate(indexes_coords.items()):
+        partial_indexes_coords.update([index_coords])
+
+        if i == len(dest_dims) - 1:
+            break
+
+        partial_dest_dims += [dest_dims[i]]
+        other_dims = dest_dims[i + 1 :]
+
+        s_partial_dest_dims = {dim for dims in partial_dest_dims for dim in dims}
+        s_other_dims = {dim for dims in other_dims for dim in dims}
+
+        if not s_partial_dest_dims.intersection(s_other_dims):
+            # this interpolation is orthogonal to the rest
+
+            yield partial_indexes_coords
+
+            partial_dest_dims = []
+            partial_indexes_coords = {}
+
+    yield partial_indexes_coords
